/*==============================================================================
 The Fraclet Programming Model - Fractal Component Model (fractal@objectweb.org)
 Copyright (C) 2002-2006 INRIA Futurs / LIFL
 Copyright (C) 2006-2007 Universitetet i Oslo

 This library is free software; you can redistribute it and/or modify it under 
 the terms of the GNU Lesser General Public License as published by the Free 
 Software Foundation; either version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful, but WITHOUT ANY 
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License along 
 with this library; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 Initial developer(s): Romain Rouvoy
 Contributor(s): 
 =============================================================================*/
package org.objectweb.fractal.adl.spoonlet.definition;

import static com.google.common.base.Join.join;
import static com.google.common.collect.Iterables.transform;
import static com.google.common.collect.Lists.newArrayList;
import static java.util.logging.Logger.getLogger;
import static org.objectweb.fractal.fraclet.types.Constants.EMPTY;

import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.logging.Logger;

import com.google.common.base.Function;

/**
 * Structure used to store the list of arguments associated to a component.
 * 
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
public class Arguments extends HashMap<String, String> {
    private static final String ARGUMENT_SEPARATOR = ",";
    private static final String ARGUMENT_AFFECTATION = "=>";
    private final Logger log = getLogger("spoonlet.fractaladl.arguments");
    private final List<String> mandatory = newArrayList();

    /* (non-Javadoc)
     * @see java.util.HashMap#put(java.lang.Object, java.lang.Object)
     */
    @Override
    public String put(String key, String value) {
        if (this.mandatory.contains(key))
            log.warning("Argument \"" + key
                    + "\" is already declared as a mandatory argument.");
        if (this.containsKey(key))
            log.warning("Argument \"" + key
                    + "\" is already declared as an optional argument.");
        if (value == null || EMPTY.equals(value))
            this.mandatory.add(key);
        else
            super.put(key, value);
        return value;
    }

    /* (non-Javadoc)
     * @see java.util.HashMap#isEmpty()
     */
    public boolean isEmpty() {
        return super.isEmpty() && mandatory.isEmpty();
    }

    private static final Function<Entry<String, String>, String> entryToString = new Function<Entry<String, String>, String>() {
        public String apply(Entry<String, String> e) {
            return e.getKey() + "=" + e.getValue();
        }
    };

    /* (non-Javadoc)
     * @see java.util.AbstractMap#toString()
     */
    public String toString() {
        StringBuffer out = (StringBuffer) join(new StringBuffer(),
                ARGUMENT_SEPARATOR, this.mandatory);
        return join(out, ARGUMENT_SEPARATOR,
                transform(entrySet(), entryToString)).toString();
    }

    private static final Function<String, String> exportString = new Function<String, String>() {
        public String apply(String s) {
            return s + ARGUMENT_AFFECTATION + asArgument.apply(s);
        }
    };

    private static final Function<Entry<String, String>, String> exportEntry = new Function<Entry<String, String>, String>() {
        public String apply(Entry<String, String> e) {
            return e.getKey() + ARGUMENT_AFFECTATION + asArgument.apply(e.getKey());
        }
    };

    /**
     * Exports the arguments as a list of parameters.
     * 
     * @return the formatted list of parameters.
     */
    public String export() {
        StringBuffer out = (StringBuffer) join(new StringBuffer(),
                ARGUMENT_SEPARATOR, transform(mandatory, exportString));
        return join(out, ARGUMENT_SEPARATOR, transform(entrySet(), exportEntry))
                .toString();
    }

    /**
     * Provides the reference to an argument.
     * 
     * @param arg the argument to reference.
     * @return the reference.
     */
    public static final Function<String, String> asArgument = new Function<String, String>() {
        public String apply(String arg) {
            return "${" + arg + "}";
        }
    };
}