/*==============================================================================
 The Fraclet Programming Model - Fractal Component Model (fractal@objectweb.org)
 Copyright (C) 2002-2006 INRIA Futurs / LIFL
 Copyright (C) 2006-2007 Universitetet i Oslo

 This library is free software; you can redistribute it and/or modify it under 
 the terms of the GNU Lesser General Public License as published by the Free 
 Software Foundation; either version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful, but WITHOUT ANY 
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License along 
 with this library; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 Initial developer(s): Romain Rouvoy
 Contributor(s): 
 =============================================================================*/
package org.objectweb.fractal.xml;

import static java.util.logging.Level.FINE;
import static java.util.logging.Level.SEVERE;
import static java.util.logging.Logger.getLogger;

import java.io.File;
import java.io.PrintWriter;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import org.objectweb.fractal.document.DocumentVisitor;
import org.objectweb.fractal.document.ElementVisitor;

/**
 * Dumps the representation of a Fractal ADL description into an XML file.
 * 
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
public class XMLDumperVisitor implements DocumentVisitor, ElementVisitor {
    private static final String TAB = "    ";
    
    private final Logger log = getLogger("spoonlet.fractaladl.definition.dumper");
    private File spoonDir;
    private PrintWriter output;
    private int incr = 0;

    /**
     * Constructor
     * 
     * @param output output base directory hosting the generated files.
     */
    public XMLDumperVisitor(File output) {
        this.spoonDir = output;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.adl.xml.DocumentVisitor#startDocument(java.lang.String, java.lang.String)
     */
    public ElementVisitor startDocument(String directory, String name,
            String header) {
        if (log.isLoggable(FINE))
            log.fine("FractalADL - Generating " + name + "...");
        try {
            File dir = new File(spoonDir, directory);
            if (!dir.exists())
                dir.mkdirs();
            File file = new File(dir, name + ".fractal");
            this.output = new PrintWriter(file);
            this.output.println(header);
            this.output
                    .println("\n<!-- Fractal ADL descriptor generated by Fraclet -->");
            this.incr = 0;
            return this;
        } catch (Exception e) {
            if (log.isLoggable(SEVERE))
                log.log(SEVERE, "File creation failure", e);
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.adl.xml.DocumentVisitor#endDocument(java.lang.String)
     */
    public void endDocument() {
        this.output.close();
        this.output = null;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.adl.xml.ElementVisitor#startElement(java.lang.String, java.util.Map, boolean)
     */
    public ElementVisitor startElement(String name,
            Map<String, Object> attributes, boolean hasChild) {
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < incr; i++)
            buffer.append(TAB);
        buffer.append("<" + name);
        for (Entry<String, Object> arg : attributes.entrySet())
            buffer.append(" " + arg.getKey() + "=\"" + arg.getValue() + "\"");
        buffer.append(hasChild ? ">" : "/>");
        this.output.println(buffer.toString());
        this.incr++;
        return this;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.adl.xml.ElementVisitor#endElement(java.lang.String, boolean)
     */
    public void endElement(String name, boolean hasChild) {
        this.incr--;
        if (hasChild) {
            StringBuffer buffer = new StringBuffer();
            for (int i = 0; i < incr; i++)
                buffer.append(TAB);
            buffer.append("</" + name + ">");
            this.output.println(buffer.toString());
        }
    }
}